/* ========================================================================= *
 * File parsing and Main Function

 * ------------------------------------------------------------------------- *
 * NOM
 *      quantizer
 * SYNOPSIS
 *      quantizer inputImg k outputName
 * DESCIRPTION
 *      Quantizes the input image on k levels and save it.
 * USAGE
 *      ./quantizer lena.pgm 4 lena_4.pgm
 *          Will compress the image lena.pgm on 4 levels and save it under
 *          the name "lena_4.pgm".
 * ------------------------------------------------------------------------- *
 * ========================================================================= */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include "PortableGrayMap.h"
#include "ImageQuantizer.h"



int main(int argc, char** argv)
{
    // Checking arguments
    if (argc != 4)
    {
        /*
         * argv[0]: name of the executable
         * argv[1]: name of the input file
         * argv[2]: number of levels
         * argv[3]: name of the output file
         */
        printf("Usage: %s <PGM input image> <int> <PGM output name>\n", argv[0]);
        return -1;
    }

    // Parsing arguments
    int argv2 = atoi(argv[2]);
    if (argv2 <= 0)
    {
        fprintf(stderr, "Invalid number of levels. Expected int > 0, found %d\n", argv2);
    }
    size_t nbLevels = (size_t) argv2;


    // Loading input Image
    PortableGrayMap* inputImg = createImageFromFile(argv[1]);
    if(!inputImg)
    {
        fprintf(stderr, "Error while loading input image\n");
        return -1;
    }

    // Quantizing
    PortableGrayMap* outputImg = quantizeGrayImage(inputImg, nbLevels);
    if(!outputImg)
    {
        fprintf(stderr, "Error while computing the reduction\n");
        deleteImage(inputImg);
        return -2;
    }

    // Computing the error
    unsigned long error = 0;
    long errComp = 0;
    for (size_t i = 0; i < inputImg->height; i++)
    {
        for(size_t j = 0; j < inputImg->width; j++)
        {
            errComp = (long)(inputImg->array[i][j]) - (long)(outputImg->array[i][j]);
            error += (unsigned long)(errComp*errComp);
        }
    }
    printf("Compression error: %lu\n", error);


    // Saving output image
    if(saveImageToFile(outputImg, argv[3]) != 0)
    {
        fprintf(stderr, "Error while saving the image\n");
        deleteImage(inputImg);
        deleteImage(outputImg);
        return -3;
    }

    deleteImage(inputImg);
    deleteImage(outputImg);
    return 0;
}
